@@ -16,7 +16,7 @@ use bitcoin::blockdata::block::{Block, BlockHeader};
 use bitcoin::blockdata::transaction::Transaction;
 use bitcoin::consensus::Decodable;
 
-pub const DEFAULT_ENDPOINT: &str = "http://localhost:8332/rest/";
+pub const DEFAULT_ENDPOINT: &str = "http://localhost:8332/rest";
 
 #[derive(Debug, Clone, Deserialize)]
 pub struct Softfork {
@@ -114,7 +114,7 @@ pub fn new(endpoint: &str) -> Context {
 impl Context {
     /// Call the REST endpoint and parse it as a JSON.
     pub async fn call_json<T: for<'de> Deserialize<'de>>(&self, path: &str) -> Result<T, reqwest::Error> {
-        let url = String::new() + &self.endpoint + path + ".json";
+        let url = format!("{}/{}.json", &self.endpoint, path);
         let result = self.client.get(url)
             .send().await?
             .json::<T>().await?;
@@ -122,7 +122,7 @@ impl Context {
     }
     /// Call the REST endpoint (binary).
     pub async fn call_bin(&self, path: &str) -> Result<bytes::Bytes, reqwest::Error> {
-        let url = String::new() + &self.endpoint + path + ".bin";
+        let url = format!("{}/{}.bin", &self.endpoint, path);
         let result = self.client.get(url)
             .send().await?
             .bytes().await?;
@@ -130,7 +130,7 @@ impl Context {
     }
     /// Call the REST endpoint (hex).
     pub async fn call_hex(&self, path: &str) -> Result<String, reqwest::Error> {
-        let url = String::new() + &self.endpoint + path + ".hex";
+        let url = format!("{}/{}.hex", &self.endpoint, path);
         let mut result = self.client.get(url)
             .send().await?
             .text().await?;
@@ -140,26 +140,22 @@ impl Context {
     }
     /// Call the [/tx](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#transactions) endpoint.
     pub async fn tx(&self, txhash: &Txid) -> Result<Transaction, Error> {
-        let path = String::from("tx/") + &txhash.to_string();
-        let result = self.call_bin(&path).await?;
+        let result = self.call_bin(&["tx", &txhash.to_string()].join("/")).await?;
         Ok(Transaction::consensus_decode(result.as_ref())?)
     }
     /// Call the [/block](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blocks) endpoint.
     pub async fn block(&self, blockhash: &BlockHash) -> Result<Block, Error> {
-        let path = String::from("block/") + &blockhash.to_string();
-        let result = self.call_bin(&path).await?;
+        let result = self.call_bin(&["block", &blockhash.to_string()].join("/")).await?;
         Ok(Block::consensus_decode(result.as_ref())?)
     }
     /// Call the [/block/notxdetails](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blocks) endpoint.
     pub async fn block_notxdetails(&self, blockhash: &BlockHash) -> Result<BlockHeader, Error> {
-        let path = String::from("block/notxdetails/") + &blockhash.to_string();
-        let result = self.call_bin(&path).await?;
+        let result = self.call_bin(&["block", "notxdetails", &blockhash.to_string()].join("/")).await?;
         Ok(BlockHeader::consensus_decode(result.as_ref())?)
     }
     /// Call the [/headers](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blockheaders) endpoint.
     pub async fn headers(&self, count: u32, blockhash: &BlockHash) -> Result<Vec<BlockHeader>, Error> {
-        let path = String::from("headers/") + &count.to_string() + "/" + &blockhash.to_string();
-        let result = self.call_bin(&path).await?;
+        let result = self.call_bin(&["headers", &count.to_string(), &blockhash.to_string()].join("/")).await?;
         let mut ret = Vec::new();
         const BLOCK_HEADER_SIZE: usize = 80usize;
         let mut offset = 0;
@@ -171,8 +167,7 @@ impl Context {
     }
     /// Call the [/blockhashbyheight](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#blockhash-by-height) endpoint.
     pub async fn blockhashbyheight(&self, height: u32) -> Result<BlockHash, Error> {
-        let path = String::from("blockhashbyheight/") + &height.to_string();
-        let result = self.call_bin(&path).await?;
+        let result = self.call_bin(&["blockhashbyheight", &height.to_string()].join("/")).await?;
         Ok(BlockHash::consensus_decode(result.as_ref())?)
     }
     /// Call the [/chaininfo](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#chaininfo) endpoint.
@@ -181,15 +176,16 @@ impl Context {
         Ok(result)
     }
     /// Call the [/getutxos](https://github.com/bitcoin/bitcoin/blob/master/doc/REST-interface.md#query-utxo-set) endpoint.
-    pub async fn getutxos(&self, checkmempool: bool, txids: &Vec<Txid>) -> Result<UtxoData, Error> {
-        let mut url = String::from("getutxos/");
+    pub async fn getutxos(&self, checkmempool: bool, txids: &[Txid]) -> Result<UtxoData, Error> {
+        let mut path = Vec::with_capacity(1 + if checkmempool { 0 } else { 1 } + txids.len());
+        path.push("getutxos".to_string());
         if checkmempool {
-            url += "checkmempool/"
+            path.push("checkmempool".to_string());
         }
-        for i in 0..txids.len() {
-            url += &(txids[i].to_string() + "-" + &i.to_string());
+        for (i, txid) in txids.iter().enumerate() {
+            path.push([txid.to_string(), i.to_string()].join("-"));
         }
-        let result: UtxoData = self.call_json(&url).await?;
+        let result: UtxoData = self.call_json(&path.join("/")).await?;
         Ok(result)
     }
 }
@@ -200,7 +196,7 @@ mod tests {
     use super::*;
     #[tokio::test]
     async fn reqwest_fail() {
-        let rest = new("http://invalid-url/");
+        let rest = new("http://invalid-url");
         assert!(rest.blockhashbyheight(0).await.is_err());
     }
     struct Fixture {
